@@ -189,6 +189,8 @@ class BaseRun(DataMixin, models.Model):
 
     @cached_property
     def ended(self):
+        if self.completed:
+            return True
         instances = self.instances.count()
         return (
             instances > 0
@@ -268,11 +270,17 @@ class BaseRun(DataMixin, models.Model):
                 started += 1
         return started
 
+    def _get_app_setting(self, key):
+        app_setting = getattr(apps.get_app_config("simpl"), key)
+        if callable(app_setting):
+            app_setting = app_setting(self)
+        return app_setting
+
     @property
     def continuous_configurable(self) -> bool:
         if self.game:
             return self.game.continuous is None
-        return apps.get_app_config("simpl").CONTINUOUS_CONFIGURABLE
+        return self._get_app_setting("CONTINUOUS_CONFIGURABLE")
 
     @property
     def continuous_open(self) -> bool:
@@ -291,6 +299,22 @@ class BaseRun(DataMixin, models.Model):
             )
         )
 
+    @property
+    def use_status_prepare(self) -> bool:
+        return self._get_app_setting("USE_STATUS_PREPARE")
+
+    @property
+    def use_status_debrief(self) -> bool:
+        return self._get_app_setting("USE_STATUS_DEBRIEF")
+
+    @property
+    def can_end(self) -> bool:
+        return self._get_app_setting("CAN_END_RUN")
+
+    @property
+    def can_restart(self) -> bool:
+        return self.can_end and self._get_app_setting("CAN_RESTART_RUN")
+
 
 class Run(BaseRun, DataMixin, models.Model):
     """
@@ -359,6 +383,14 @@ class BaseInstance(DataMixin, models.Model):
     def running(self) -> bool:
         return bool(self.date_start and not self.date_end)
 
+    def stop(self):
+        self.date_end = timezone.now()
+        self.save()
+
+    def restart(self):
+        self.date_end = None
+        self.save()
+
     def get_status_display(self):
         if not self.date_start:
             return "Unstarted"
@@ -537,9 +569,14 @@ class Player(BasePlayer):
 
 class APIToken(models.Model):
     token = models.CharField(max_length=32, editable=False, unique=True)
+    name = models.CharField(max_length=100, blank=True)
     last_used = models.DateField(blank=True, null=True)
 
     def __str__(self):
+        return self.name or self.token_part
+
+    @property
+    def token_part(self):
         return f"{self.token[:8]}..."
 
     def save(self, *args, **kwargs):
